import java.util.*;

/**
 * Superclass of our various Comparator objects.  The instantiation
 * of a child of this class will specify: <BR>
 * 1. compare()  (per the Comparator interface requirements) <BR>
 * 2. doubleValue()  Returns the corresponding value of the from the FinchSensor <BR>
 * 3. toString()  Returns a string that describes the child
 * 
 * <P>
 * 
 * @author Andrew H. Fagg and Steven Ness
 *
 */
public abstract class SensorComparator implements Comparator<FinchSensor> {
	/**
	 * Quick method used to compare integers a and b
	 * <P>
	 * @param a
	 * @param b
	 * @return	1 if <code>a</code> is greater than <code>b</code>, -1 if
	 * 				the reverse is true, and 0 if they are equal
	 */
	static protected int compare(int a, int b){
		if(a > b) return(1);
		if(a == b) return(0);
		return(-1);
	}
	
	/**
	 * Quick method used to compare doubles
	 * <P>
	 * @param a
	 * @param b
	 * @return	1 if <code>a</code> is greater than <code>b</code>, -1 if
	 * 				the reverse is true, and 0 if they are equal
	 */
	static protected int compare(double a, double b){
		if(a > b) return(1);
		if(a == b) return(0);
		return(-1);
	}
	
	/**
	 * Return the sum over an array of ints
	 * 
	 * @param vals	Array of integers
	 * @return	Sum over the array.
	 */
	
	static protected int sum(int vals[]){
		int out = 0;
		for(int i=0; i < vals.length; ++i){
			out += vals[i];
		}
		return(out);
	}
	

	/**
	 * Compute the magnitude (or length) of a vector (specifically, the L2 norm)
	 * <P>
	 * 
	 * @param vec	An array of doubles
	 * @return		The vector magnitude
	 */
	
	static protected double arrayMagnitude(double[] vec){
		double mag = 0.0;
		
		for(int i = 0; i < vec.length; ++i){
			mag += vec[i]*vec[i];
		}
		return(Math.sqrt(mag));
	}

	/**
	 * Return the value of obj that is specified by the instantiation of the child
	 * class.
	 * 
	 * @param obj	A FinchSensor sample
	 * @return		The value from obj that is specified by the child class
	 */
	
	public abstract double doubleValue(FinchSensor obj);
	
	/**
	 * Force the child class to return a string that describes itself. 
	 * 
	 * @return The self-describing string.
	 */
	
	public abstract String toString();
	
}
