import finch.Finch;
import java.util.*;


/**
 * The main driver for Lab 4
 * <P>
 * <dt> <b>The responsibilities are: </b>
 * <dd>  Read a sequence of sensor samples from the Finch
 * <dd>  Compute the mean across the set of samples for some sensor
 * <dd>  Find the min, median and max samples for some sensor and report the sample
 * <dd>  Repeat for different sensor streams  (keys)
 * </dt>
 * 
 * <P>
 * 
 * @author Andrew H. Fagg and Steven Ness
 */

public class sensorDriver {
	
	/** Total number of samples to take */
	public static final int numSamples = 100;
	
	/** 
	 * Interval between samples (note that there is also other overhead that extends
	 * this delay)
	 */
	public static final int sampleInterval = 50; // In ms

	/**
	 * Return the sensor sample that is the median value of the key used to
	 *   sort a set.
	 *   
	 * <P>
	 * Algorithm: <BR>
	 * The median sample is the middle one given that the set has already been ordered.
	 * <P>
	 * <dt><b>Conditions:</b>
     * <dd>PRE  -         The log must already be instantiated and sorted according to the key
     * <dd>POST -         n/a
	 * <P>
	 * 
	 * @param log	A set of FinchSensors, already sorted by some key
	 * @return		The sensor element in the middle of the ordered list
	 *                    (this is the MEDIAN sample)
	 *
	 */
	
	static public FinchSensor median(List<FinchSensor> log)
	{
		// TODO: Fill in a real return value
		return(log.get(0));
	}
	
	/**
	 * Compute the mean value of a some sensor value across a set of FinchSensors.
	 *   The particular sensor stream is specified by the SensorComparator
	 * <P>
	 * Algorithm:
	 * Return the sum of the channel values divided by the number of samples.
	 * <P>
	 * 
	 * @param log	A set of FinchSensors
	 * @param comp	A SensorComparator that specifies the key
	 * @return	The mean value of the keys
	 */
	
	static public double mean(List<FinchSensor> log, SensorComparator comp)
	{
		double out = 0.0;
		
		// Iterate over all all sensors
		
		// TODO: Fill in this iteration
		
		
		return(out/log.size());
	}
	
	/**
	 * For the given sensor channel (as indicated by comp), display:
	 * <dt>
	 * <dd> The mean value of this sensor channel
	 * <dd> the entire sensor sample for the minimum, median and 
	 * maximum channel values
	 * </dt>
	 * <P>
	 * Algorithm: <BR>
	 * 1.  Sort the elements of log according to comp <BR>
	 * 2.  Report the mean of the sensor value <BR>
	 * 3.  Given the ordering of the sorted list, we can pull out the 
	 * min, median and max sensor samples by extracting the first, middle and 
	 * last elements in the list.
	 * 
	 * <P>
	 * 
	 * @param log	A valid list of Finch sensor samples
	 * @param comp	The sensor channel on which to organize the list
	 * 
	 * <dt><b>Conditions:</b>
     * <dd>PRE  -         The log must already be instantiated
     * <dd>POST -         The order of log has changed so that it is ordered according to the key
     * </DT>
	 */
	public static void displayStats(List<FinchSensor> log, SensorComparator comp)
	{
		// Sort the list by a particular variable in FinchSensor: smallest to largest
		Collections.sort(log, comp);
		
		// Name of the variable
		System.out.println("\nVariable: " + comp);
		
		// The mean sample
		System.out.println("Mean Value: " + mean(log, comp));
		
		// The minimum sample: the first one in the sorted list
		System.out.println("Min:    " + log.get(0)); 
		
		// The median sample: no loner makes sense since this is a set and not a list.
		System.out.println("Median: " + median(log));	
		
		// The max sample: the last one in the list
		System.out.println("Max:    " + log.get(log.size()-1));
		
		System.out.println("");
	}
	
	/**
	 * Display the sequence of sensor samples in the given List
	 * <P>
	 * Algorithm: <BR>
	 * -  Loop over each sensor in the List and display it
	 * <P>
	 * 
	 * 
	 * @param sensors	A List of sensor samples
	 */
	
	public static void displaySensors(List<FinchSensor> sensors)
	{
		// Loop over all elements in the list
		for(FinchSensor sensor: sensors)
		{
			System.out.println(sensor);
		}
	}
	
	/**
	 * Read a sequence of samples from the Finch sensors
	 * 
	 * @param 	myFinch		Reference to the finch object
	 * @return		A List of FinchSensor samples
	 */
	
	public static List<FinchSensor> ReadSensors(Finch myFinch)
	{
		List<FinchSensor> samples = new ArrayList<FinchSensor>();
		
		// Loop over the set of samples
		for(int i = 0; i < numSamples; ++i){
			// Grab the sensor values
			int light[] = myFinch.getLightSensors();
			double acceleration[] = myFinch.getAccelerations();
			double temperature = myFinch.getTemperature();
			boolean obstacle[] = myFinch.getObstacleSensors();
			
			// Create a new FinchSensor
			samples.add(new FinchSensor(light, acceleration, obstacle, temperature));
			
			// Delay before the next sample
			myFinch.sleep(sampleInterval);
			
		}
		return(samples);
	}
	
	/**
	 * The main method
	 * 
	 * Algorithm: <BR>
	 * 1.  Connect to the Finch <BR>
	 * 2.  Take a log of sensor readings <BR>
	 * 3.  Compute statistics over the log <BR>
	 * 4.  Close down connection to the Finch <BR>
	 *
	 */
	
	public static void main(String[] args)
	{
		// Connect to the Finch
		Finch myFinch = new Finch();
		
		// Log a set of samples
		myFinch.setLED(255,0,0); 
		List<FinchSensor> log = ReadSensors(myFinch);
		myFinch.setLED(0,0,255); 
	
		// Show the samples in order
		System.out.println("Original Log:");
		displaySensors(log);
		
		// Show the samples with the min,median,max values of Z component of acceleration
		displayStats(log, new ZaccelComparator());
		
		// Show the samples with the min,median,max values of temperature
		displayStats(log, new TemperatureComparator());
		
		// Show the samples with the min,median,max values of magnitude of acceleration
		displayStats(log, new AccelComparator());
		
		// Show the samples with the min,median,max values of the left light sensor
		displayStats(log, new LightComparator());
		
		// Clean up and quit
		myFinch.stopWheels();
		myFinch.quit();
	}
}